coordROC <- structure(function(  #ROC coordinates
### Returns the ROC coordinates (FPR, TPR) of a classifier relatively to a reference one.
                               betaHat,
### numeric vector; the input classifier.
                               beta
### numeric vector; the reference classifier with the same length as the input classifier.
                               ){
  ech.sel <- which(betaHat!=0)
  if (length(betaHat)==0) {
    TPR <- NA
    FPR <- NA
  } else if (is.element(NaN, betaHat)){
    TPR <- NA
    FPR <- NA
  } else {
    P.true <- sum(beta!=0)
    l <- length(beta)
    if (is.element(NA, ech.sel)) {
      TPR <- NA
      FPR <- NA
    } else {
      TP <- length(which(beta[ech.sel]!=0))
      FP <- length(ech.sel) - TP
      TN <- l-P.true-FP
      FN <- P.true-TP
      FPR <- 1-(TN/(TN + FP))
      TPR <- TP/(TP+FN)
    }
  }
  ##value<< A vector with the ROC coordinates
  c(FPR=FPR, ##<< false positive rate (1-specificity)
    TPR=TPR ##<< true positive rate (sensitivity)
    )
}, ex=function(){
  set.seed(5)
  ref <- sample(c(0,1), 30, replace=TRUE)
  ref
  coordROC(rep(0, 30), ref)
  coordROC(rep(1, 30), ref)
  c <- sample(c(0,1), 30, replace=TRUE)
  c
  coordROC(c, ref)
})

evalRoc <- structure(function(
### Returns, as an array, the ROC coordinates of a set of methods given the regression coefficients of these methods and the 'true' (simulated) coefficients.
                              estBetas,
### numeric array with 4 dimensions containing the estimated regression coefficients. The first 2 dimensions correspond to the regularization parameter and the number of variables. The last 2 dimensions correspond to the regression method and the simulation run and must be called respectively "methods" and "simulation".
                              trueBetas,
### (number of variables) x (number of simulations) matrix containing the reference regression coefficients. 
                              aggregate=TRUE
### logical. Indicates if the ROC coordinates returned should be aggregated over the simulation runs.
                              ){
  dimnames <- dimnames(estBetas)
  methods <- dimnames(estBetas)$methods
  simu <- dimnames(estBetas)$simulation
  dimnames <- list(ROC=c("FPR", "TPR"),
                   grid=dimnames$grid,
                   methods=methods,
                   simulation=simu)
  dims <- sapply(dimnames, length)
  coords <- array(NA, dim=dims, dimnames=dimnames)
  for (jj in seq(along=simu)) {
    trueBeta <- trueBetas[, jj]
    for (method in methods) {
      coefMat <- estBetas[,, method, jj]
      coords[,, method, jj] <- apply(coefMat, 1, coordROC, trueBeta)   
    }
  }
  if (aggregate) {
    coords <- apply(coords, 1:3, mean, na.rm=TRUE)
  }
  ##seealso<< \code{\link{coordROC}}.
  ##value<< A numeric array containing the 3 dimensions: "ROC" (ROC coordinates), "grid" (regularization parameter grid) and "methods" (regression methods names). If aggregate=FALSE, the 4th dimension "simulation" is kept.
  coords
  
}, ex=function(){
  nb.simu <- 5
  
  set.seed(5)
  blockSizes <- sample(2:5, 8, replace=TRUE)
  p <- sum(blockSizes)
  sig.blocks <- 2
  nb.per.block <- 1
  BS <- replicate(nb.simu, simBeta(blockSizes, sig.blocks, 
                                   nb.per.block), simplify=FALSE)
  N <- 10
  corr <- 0.5
  r2 <- 0.7
  datList <- lapply(BS, FUN=function(x) {
    betaSNP <- x$betaMat[, "betaSNP"]
    simulation(N, betaSNP, x$blockSizes, corr, r2)
  })
  trueBetaSNP <- sapply(BS, function(simb) {simb$betaMat[, "betaSNP"]})
  nlambda <- 10
  dimnames <- list(
                   grid=seq(length=nlambda),
                   predictor=1:p,
                   methods=c("GL", "lasso"),
                   simulation=seq(length=nb.simu))
  dims <- sapply(dimnames, length)
  coefs <- array(NA, dim=dims, dimnames=dimnames)
  
  for (jj in 1:nb.simu){
    datJJ <- datList[[jj]]
    X <- datJJ$X
    stopifnot(ncol(X)==p)
    y <- datJJ$y
    blockSizes <- BS[[jj]]$blockSizes
    nBlocks <- length(blockSizes)
    groups <- rep(1:nBlocks, times=blockSizes)
    coefs[,, "GL", jj] <- select("groupLasso", X, y, nlambda=nlambda, groups=groups)
    coefs[,, "lasso", jj] <- select("lasso", X, y, nlambda=nlambda)
  }
  roc <- evalRoc(coefs, trueBetaSNP)
  str(roc)
})
