plotHeatmap <- structure(function(
###  Heat map of LD blocks where SNPs/LD blocks are highlighted
                                  X,
### genotype matrix coded in {1, 2, 3}. The SNPs to be plotted in columns.
                                  blockSizes,
### a vector of integers specifying the sizes of the groups. The sum of the sizes must be equal to the length of \code{ncol(p)}. 
                                  selBlocks=NULL,
### a vector of integers specifying the indices of the blocks to be highlighted among \code{blockSizes}. Default \code{NULL}.
                                  stats="R.squared",
### a character specifying the LD measure to be used. This should contain one of these strings: \code{R.squared} or \code{D.prime}. Defaults to \code{R.squared}.
                                  snpNames=NULL,
### character. Names of the SNPs to be plotted. Defaults to NULL.
                                  snpStar=NULL){
### a vector of character. Names of the SNPs to be highlighted. Must be element of \code{snpNames}. Defaults to NULL.
  
  ##references<<J.-H. Shin and S. Blay and B. McNeney and J. Graham. LDheatmap: An R Function for Graphical Display of Pairwise Linkage Disequilibria Between Single Nucleotide Polymorphisms. Journal of Statistical Software 16: Code Snippet 3.
  ##seealso<<\code{\link[snpStats]{ld}}, \code{\link[LDheatmap]{LDheatmap}}.
  
  p <- ncol(X)
  if (is.null(snpNames)){
    snpNames <- as.character(1:p)
  }
  
  dn <- rep("\n", p)
  if (!(is.null(snpStar))){
    idxStar <- which(is.element(snpNames, snpStar))
    dn[idxStar] <-  snpNames[idxStar]
  }
  
  ##details<<The LD measures are calculated using the function \code{\link[snpStats]{ld}} which handles missing values of the genotype.
  LD <- ld(.snpM(X), depth=p-1, stats=stats, symmetric=TRUE)
  diag(LD) <- 1
  
  MyHeatmap <- LDheatmap(as.matrix(LD), color=gray(0:29/31), add.key=TRUE, add.map=FALSE)
  attr(MyHeatmap$LDmatrix, "dimnames")[[1]] <-  dn
  LDheatmap(MyHeatmap, SNP.name = dn, add.map=FALSE)

  cs <- cumsum(c(1, blockSizes))
  for (ii in 1:length(blockSizes)){
    if (blockSizes[ii]!=1){
      if (is.element(ii, selBlocks)) {
        LDheatmap.highlight(MyHeatmap, i=cs[ii], j=cs[ii+1]-1, col="red")
      } else {
        LDheatmap.highlight(MyHeatmap, i=cs[ii], j=cs[ii+1]-1, col="black")
      }
    }
  }
  if (!(is.null(snpStar))){
    LDheatmap.marks(MyHeatmap, idxStar,  idxStar,  pch="*", gp=gpar(cex=2.5, col="red"))
  }
  grid.edit(gPath("ldheatmap", "SNPnames"), gp = gpar(cex=1.1))
###A heatmap plot of the LD blocks and optionally a highlight of blocks/SNPs.  
}, ex=function(){
  ## Structured data: 7 clusters
  N <- 100
  p <- 30
  r2 <- 0.2
  corr <- 0.8
  blockSizes <- c(2:7, 3)
  sig.blocks <- 5
  nb.per.block <- 1
  betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
  sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2)
  X <- sim$X
  plotHeatmap(X, betas$blockSizes)
  plotHeatmap(X, betas$blockSizes, stats="D.prime", snpNames=as.character(1:30),
              snpStar=as.character(c(1,3,15,20)))
})


plotGroupsGL <- structure(function(  # significance plot of LD blocks
### This function provides a graphical display for interpreting selected groups in function of univariate p-values of the SNPs contained in these blocks. 
                                  coefsGL,
### should be an output of the function select with method="groupLasso" : real matrix of the regression coefficients of the Group Lasso with an attribute \code{"groups"} describing the group structure of the variables. See \code{\link{select}}. 
                                  nbGroup,
### integer. Indicates the number of groups to be plotted. Must be less or equal to the total number of groups.
                                  pvals,
### numeric vector. Univariate p-values of the variables. Must be the same length as \code{ncol(coefsGL)}.
                                  meth="BH",
### adjustment method for univariate \code{p}-values. See \code{\link[stats]{p.adjust}}.
                                  t=0.25
### significance threshold. 
                                  ){
  groups <- attr(coefsGL, "groups")
  resList <- apply(coefsGL, 1, FUN=function(beta) which(beta!=0))
  grpList <- lapply(resList, FUN=function(idx) {
    unique(groups[idx])
  })
  
  grpListU <- unique(grpList)
  l <- sapply(grpListU, length)
  id <- which(l>=nbGroup)
  grpListU <- grpListU[1:min(id)]

  
  grpOrder <- NULL
  for (ii in 1:length(grpListU)) {
    grp <- na.omit(grpListU[[ii]])
    grpOrder <- c(grpOrder, setdiff(grp, grpOrder))
  }
  
  pvalList <- lapply(grpOrder, FUN=function(grp) {
    pvals[which(groups==grp)]
  })
  
  logScale <- TRUE
  
  step <- 1/nbGroup
  if (logScale) {
    hist(-log10(pvals), breaks=100, border="gray", prob=TRUE, xlab=expression(-log[10](p)), main="")
    adjp <- p.adjust(pvals, method=meth)
    abline(v=min(-log10(pvals)[which(adjp<=t)]), lty=2)
    abline(v=-log10(0.05), lty=4)
  } else {
    hist(pvals, breaks=100, border="gray", prob=TRUE, xlim=c(0, 0.2))
  }
  for (ii in 1:nbGroup) {
    pv <- pvalList[[ii]]
    lwd <- min(3/quantile(pv, 0.5)/40, 6)
    if (logScale) {
      pv <- -log10(pv)
    }
    yy <- ii*step
    segments(x0=min(pv), x1=max(pv), y0=yy, y1=yy, col=ii, lwd=lwd)
    segments(x0=pv, x1=pv, y0=yy-step/4, y1=yy+step/4)
  }

  ##seealso<< \code{\link{select}}, \code{\link{grplassoCWard}}, \code{\link[stats]{p.adjust}}
  ##value<< The graphical display returned contains, in the background, the histogram of the distribution of the (-log10-transformed) univariate p-values. It also represents each of the first \code{nbGroup} selected groups by a colored horizontal segment ranging from the smallest to the largest univariate p-value of the block. Vertical black segments indicate univariate p-values of each variable in these group and a vertical line highlights the significance threshold chosen.
  
}, ex=function(){
  set.seed(0.2)
  ## Structured data: 7 clusters
  N <- 100
  p <- 30
  r2 <- 0.5
  corr <- 0.8
  blockSizes <- c(2:7, 3)
  sig.blocks <- c(3,5,7)
  nb.per.block <- 1
  betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
  sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2, minMAF=0.45)
  blockSizes <- betas$blockSizes  ## effective blockSizes used for simulation !
  nBlocks <- length(blockSizes)
  groups <- betas$betaMat[,"groups"]
  coefsGL <- select("groupLasso", sim$X, sim$y, groups=groups)
  ## univariate p-values
  pvals <- apply(sim$X, 2, FUN=function(vect){
    pv <- summary (lm(sim$y ~ vect))$coefficients[2,4]
  })
  plotGroupsGL(coefsGL, nbGroup=6, pvals)
  ## all the univariate p-values of the first selected 2 blocks are smaller than
  ## the 0.05 significance threshold !
})

##############################################################################
## HISTORY:
## 2014-07-17
## o improvement in plotGroupsGL: grpOrder calculated only for the first
## 'nbGroup' groups.
## o improvement in plotHeatmap: handles blocks of size 1.
## 2013-...
## o Created.
##############################################################################
