## seq on the log scale
.seqLog <- function(from, to, length) {
  10^(seq(log10(from), log10(to), length=length))
}


select <- structure(function( #variable selection models
### Fits the regression model for the methods Univariate, Lasso, Elastic-Net, Group Lasso.
                             method=c("lasso", "elastic.net", "groupLasso", "univ"),
### regression model.
                             X,
### genotype matrix coded in {1, 2, 3}.
                             y,
### phenotype vector.
                             nlambda=100,
### length of the vector of \eqn{l_1-}penalty levels. Defaults to 100.
                             lambda2=0.8,
### numeric value. Tunes the \eqn{l_2-}penalty of the Elastic Net. Defaults to 0.8.
                             ...
### optionally further arguments for downstream selection methods.
                             ) {
  ## Validate arguments
  method <- match.arg(method)
  p <- ncol(X)
  N <- nrow(X)
  stopifnot(length(y)==N)
  
  if (method=="lasso") {
    out <- elastic.net(X, y, nlambda1=nlambda, lambda2=0, max.feat=p)
    coefs <- as.matrix(out@coefficients)
  } else if (method=="elastic.net") {
    out <- elastic.net(X, y, nlambda1=nlambda, lambda2=lambda2, max.feat=p)
    coefs <- as.matrix(out@coefficients)
  } else if (method=="groupLasso") {
    sel <- grplassoCWard(X, y, nlambda=nlambda, ...)
    coefs <- sel$coefs
    groups <- sel$groups
    attr(coefs, "groups") <- groups
  }
  else if (method=="univ") {
    pvals <- apply(X, 2, .lmModif, y, ...)
    alpha.univ.min <- min(pvals)
    alpha.univ.max <- max(pvals)
    grille.univ <- 10^(    seq(   log10(alpha.univ.min),  log10(alpha.univ.max), length=nlambda    ))  
    coefs <- .selectUniv(pvals, grille.univ)
  } 
  ##value<< 
  coefs ##<< \code{nlambda x p} matrix of regression coefficients for penalized regression models (Group Lasso, Lasso and Elastic Net). Logical matrix for the univariate model where each row corresponds to a threshold p-value for selection. The group structure (known or estimated) is returned as an attribute.
  ##references<< R. Tibshirani. Regression shrinkage and selection via the lasso. Journal of the Royal Statistical Society. Series B (Methodological), pages 267-288, 1996.
  ##references<< H. Zou and T. Hastie. Regularization and variable selection via the elastic net. Journal of the Royal Statistical Society: Series B (Statistical Methodology), 67(2):301-320, 2005.
  ##seealso<< \code{\link{grplassoCWard}}.
}, ex=function(){

  set.seed(5)
  N <- 50
  r2 <- 0.5
  corr <- 0.5
  blockSizes <- sample(3:5, 10, replace=TRUE)
  p <- sum(blockSizes)
  p
  sig.blocks <- 4
  nb.per.block <- 1
  betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
  sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2, minMAF=0.45)
  X <- sim$X
  y <- sim$y
  blockSizes <- betas$blockSizes  ## effective blockSizes used for simulation !
  nBlocks <- length(blockSizes)
  groups <- betas$betaMat[,"groups"]
  coefsGL <- select("groupLasso", sim$X, sim$y, groups=groups)
  head(blockSizes, 3)  ## sizes of the first 3 blocks
  ## simulated regression coefficients of the first 3 blocks
  head(betas$betaMat[,"betaSNP"], sum(blockSizes[1:3]))   
  head(betas$betaMat[,"betaBl"], sum(blockSizes[1:3]))  
  which(coefsGL[2,]!=0)  ## indices of the first selected group
  coefsL <- select("lasso", sim$X, sim$y, lambda2=0.5)
  pvalsUniv <- select("univ", sim$X, sim$y)
  which(pvalsUniv[2,]!=0) ## index of the first selected variable.

})

grplassoCWard <- structure(function(
### Fits the solution of a Group Lasso problem. If the groups are not given, infers the group structure using a constrained hierarchical clustering and then estimates the number of clusters using the Gap statistic method.
                                    X,
### genotype matrix coded in {1, 2, 3}. Variables in columns.
                                    y,
### phenotype vector.
                                    nlambda=100,
### length of the vector of penalty level. Defaults to 100.
                                    groups=NULL,
### vector which defines the grouping of the variables. Components sharing the same number build a group. Non-penalized coefficients are marked with NA. If NULL (default), the groups are assessed using the function \code{gapStatistic}.
                                    ...
### optionally further arguments for the function \code{\link{gapStatistic}}.  
                                    ){
  if (is.null(groups)) {
    ## statistic of cWard as parameters
    res <- gapStatistic(X, ...)
    inferred.K <- res$best.k
    ## print(inferred.K)
    tree <- res$tree
    groups <- cutree(tree, inferred.K)
    nBlocks <- max(groups)
    blockSizesHat <- sapply(1:nBlocks, FUN=function(v, groups){
      length(which(groups==v))
    }, groups)
  }
  ## Take care of the intercept
  groups <- c(NA, (groups+1))
  X <- cbind(1, X)
  lambdamax <- lambdamax(X, y, index=groups, penscale = sqrt,
                         model = LinReg(), standardize=FALSE)
  if (is.na(lambdamax)){
    lambdamax <- 10e-6
  }
  lambda <-  .seqLog(lambdamax, lambdamax*1e-10, length=nlambda)
  res <- grplasso(X, y, index=groups, lambda=lambda,
                  model=LinReg(),
                  control=grpl.control(update.hess="lambda", trace=0),
                  standardize=FALSE)
  coefs <- t(res$coefficients[-1,])  ## Take care of the intercept
  dimnames(coefs) <- NULL
  groups <- groups[-1] ## delete the intercept !
  groups <- groups-1
  names(groups) <- NULL
  
  ##value<< A list with elements
  list(coefs=coefs, ##<< \code{nlambda x p} matrix of the Group Lasso coefficients. 
       groups=groups  ##<< provided/estimated groups.
       )
  ##seealso<< \code{\link{gapStatistic}}, \code{\link{cWard}}, \code{\link{select}}.
  ##references<< J. H. Ward Jr. Hierarchical grouping to optimize an objective function. Journal of the American statistical association, 58(301):236-244, 1963.
}, ex=function(){

  set.seed(5)
  ## Structured data: 3 clusters
  N <- 30
  r2 <- 0.2
  corr <- 0.6
  blockSizes <- c(3:5)
  p <- sum(blockSizes)
  nBlocks <- length(blockSizes)
  sig.blocks <- 5
  nb.per.block <- 1
  betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
  sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2, minMAF=0.45)
  X <- sim$X
  y <- sim$y
  groups <- betas$betaMat[,"groups"]   ## effective groups: betas$blockSizes !!
  ## Oracle Group Lasso: with "true" groups
  OGL <- grplassoCWard(X, y, groups=groups)
  identical(OGL$groups, groups)    ## same groups vector retrieved
  ## Group Lasso with estimated groups
  ## we use B=50 to keep it fast..
  B <- 50
  traceWB <- lapply(1:B, FUN=function(ii){
    getTraceW(N, p)
  })
  traceWB <- simplify2array(traceWB)
  OGE <- grplassoCWard(X, y, groups=NULL, min.nc=1, max.nc=p-1, traceWB=traceWB)
  identical(OGE$groups, groups)   ## groups well estimated !

})
