simBeta <- structure(function(  #regression vector generation
### Given a set of group-structured variables, this function allows to simulate vector of coefficients related to each variable given the sizes of the non-null groups (groups containing at least one non-null coefficient) and the number of non-null variables within each group. This function allows also to simulate a logical "group-vector", that is all the variables contained in a non-null group are simulated as TRUE.
                              blockSizes,
### vector of integers with the sizes of the groups.
                              sig.blocks,
### vector of integers. Sizes of the non-null groups. If two or more non-null groups have the same size, this value of the size should be replicated in sig.blocks. Should be a subset of \code{blockSizes}.
                              nb.per.block,
### integer or vector of integers. The number of non-null coefficients within each non-null group. If it is an integer, should be less or equal to min(\code{sig.blocks}). If it is a vector of integers, this latter should have the same length as \code{sig.blocks} and \code{resample} must be set to FALSE.
                              resample=TRUE
### logical. Indicates if the block sizes should be resampled or not.
                              ){
  nBlocks <- length(blockSizes)
  if (resample){
    blockSizes <- sample(blockSizes, nBlocks)
  }
  if (length(nb.per.block)==1){
    nb.per.block <- rep(nb.per.block, length(sig.blocks))
  }
  p <- sum(blockSizes)
  tampon <- sig.blocks
  
  betaBl <- NULL
  betaSNP <- NULL
  ii <- 0
  npb <- 0
  for (bs in blockSizes){
    signif <- bs %in% tampon
    if (signif){
      idbl <- (which(tampon==bs))[1]
      npb <- nb.per.block[idbl]
    }
    betaSNP <- c(betaSNP, rep(c(1,0), times=c(npb*signif, bs-npb*signif)))
    ##note<< Non-null coefficients are placed at the first positions of the non-null group.
    betaBl <- c(betaBl, rep(signif, bs))
    if (signif){
      tampon <- tampon[-idbl]
      nb.per.block <- nb.per.block[-idbl]
    }
    npb <- 0
  }
  signs <- replicate(p, sample(c(-1,1), 1))
  betaSNP <- betaSNP*signs

  betaMat <- cbind(betaSNP=betaSNP,   ##<< the vector of coefficients. Non-null ones are randomly sampled among the values \{-1,1\}.
                   betaBl=betaBl, ##<< logical vector. TRUE is assigned to all the variables of the non-null groups.
                   groups=rep(1:nBlocks, times=blockSizes))
  
  ## position <- quel.bloc(betaSNP, blockSizes)
  ##value<< A list with elements
  list(blockSizes=blockSizes, ##<< a resampling of the original argument \code{blockSizes} used for the simulation of the coefficients,
       betaMat=betaMat ##<< \describe{\item{a \code{p x 3} matrix with columns}{
       ##\itemize{
       ## \item{\code{betaSNP}: }{the vector of coefficients. Non-null ones are randomly sampled among the values \{-1,1\}}
       ## \item{\code{betaBl}: }{logical vector. TRUE is assigned to all the variables of the non-null groups.}
       ## \item{\code{groups}: }{the vector which defines the grouping of the variables. Components sharing the same number build a group.}
       ## }
       ## }}
       )
  ##seealso<< \code{\link{simulation}}.
}, ex=function(){
  set.seed(5)
  blockSizes <- sample(2:5, 5, replace=TRUE)
  sig.blocks <- c(2,3)
  l <- length(sig.blocks)
  if (sum(is.element(sig.blocks,blockSizes))!=l) stop("sig.blocks must be a subset of blockSizes !")
  ## one associated variable per group
  nb.per.block <- 1
  simb <- simBeta(blockSizes, sig.blocks, nb.per.block)
  betaMat <- simb$betaMat
  simb$blockSizes ## effective blockSizes used for the simulation !
  betaSNP <- betaMat[, "betaSNP"]
  betaSNP  ## the first group of the two of size 2 is considered as non-null. 

  betaBl <- betaMat[, "betaBl"]
  betaBl ## all the coefficients of the non-null group are non null.

  ## different number of associated variables per group
  ## nb.per.block should have the same length as sig.blocks !
  nb.per.block <- c(1,2)
  simb <- simBeta(blockSizes, sig.blocks, nb.per.block)
  simb$blockSizes
  simb$betaMat[,"betaSNP"]
  
})


simulation <- structure(function(  #SNP data generation
### This function simulates SNP data with genotypes coded by 1, 2 and 3 as well as a quantitative phenotype assuming group-structure among the SNPs. 
                                 N,
### integer; number of individuals.
                                 beta,
### numeric vector;  regression coefficients.
                                 blockSizes,
### a vector of integers specifying the sizes of the groups. The sum of the sizes must be equal to the length of \code{beta}.
                                 corr,
### numeric value between 0 and 1. Correlation level among the variables within each block.
                                 r2,
### numeric value between 0 and 1. Coefficient of determination
                                 minMAF=0.05,
### minimum value of the Minor Allele Frequency. Default \eqn{0.05}. See details below.
                                 maxMAF=0.5,
### maximum value of the Minor Allele Frequency. Default \eqn{0.5}. See details below.
                                 beta0=3
### numeric value; intercept coefficient. Default 3.
                                 ){
  p <- length(beta)
  Z <- matrix(0, N, p)
  count <- 0
  matList <- lapply(blockSizes, FUN=function(x) {
    m <- matrix(corr, x, x)
    diag(m) <- 1
    return(m)
  })
  for (bb in matList){
    l.bb <- ncol(bb)
    Z.bb <- matrix(rnorm(l.bb*N),N,l.bb) %*% chol(bb)
    Z[,(count+1):(count+l.bb)] <- Z.bb
    count <- count + l.bb
  }
  MAF <- runif(p, min=minMAF, max=maxMAF)
  X <- sapply(1:p, FUN=function(ii){
    maf <- MAF[ii]
    c1 <- qnorm(maf^2)
    c2 <- qnorm(maf+maf^2)
    vect <- 3*(Z[,ii]>c2) + 1*(Z[,ii]<c1) + 2*( (Z[,ii]<c2) & (Z[,ii]>c1) )
    return(vect)
  })
  
  ## X <- 3*(Z>c)+1*(Z<(-c))+2*(abs(Z)<c)
  ## details<< For all  \eqn{i \in {1,..,n}}, the vector \eqn{X_i} is generated from a p-dimensional multivariate normal distribution whose covariance matrix is block-diagonal. If \eqn{j \leq j'} are in the same group, \eqn{cov(X_j , X_{j'} ) = corr} else \eqn{cov(X_j,X_{j'}) = 0}. A vector \code{maf} of the p minor allele frequency values of each SNP is uniformly generated between minMAF and maxMAF. Then, we set \eqn{X_{ij}} to 1,2 or 3 according to whether \eqn{X_{ij} < c1}, \eqn{c1 \leq X_{ij} \leq c2} or \eqn{X_{ij} > c2} with \code{c1=qnorm(maf^2)} and \code{c2=qnorm(maf+maf^2)}.
  sigma  <- sqrt((1-r2)/r2 * t(beta) %*% var(X) %*% beta)
  epsilon <- rnorm(N) * sigma
  y <- beta0 + X %*% beta + epsilon
  r2 <- 1 - sum(epsilon^2) / sum((y-mean(y))^2)
  ##details<<The effective coefficient of determination is estimated for the simulated regression model.
  ##value<< A list with elements
  list(y=y, ##<< Vector of quantitative phenotype.
       X=X, ##<< \code{N x p} genotype matrix coded in \{1,2,3\} where \code{p=length(beta)} is the number of variables (SNPs).
       r2=r2  ##<< Estimated coefficient of determination.
       )
  ##references<<T. T. Wu, Y. F. Chen, T. Hastie, E. Sobel, and K. Lange. Genome-wide association analysis by lasso penalized logistic regression. Bioinformatics, 25(6):714-721, 2009.
  ##seealso<< \code{\link{simBeta}}.
}, ex=function(){

  set.seed(5)
  N <- 100
  r2 <- 0.3
  corr <- 0.5
  blockSizes <- sample(5:80, 10, replace=TRUE)
  p <- sum(blockSizes)
  p
  sig.blocks <- 20
  l <- length(sig.blocks)
  if (sum(is.element(sig.blocks,blockSizes))!=l) stop("sig.blocks must be a subset of blockSizes !")
  nb.per.block <- 1
  betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
  ## effective blockSizes used for simulation !
  betas$blockSizes
  betaSNP <- betas$betaMat[, "betaSNP"]
  sim <- simulation(N, betaSNP, betas$blockSizes, corr, r2)
  sim$r2 ## estimated coefficient of determination.
  ## The greater are the values of \code{N} and \code{p},
  ## the more accurate is the estimation of \code{r2}.
  table(sim$X[,1])
  ## blockwise correlation
  image(abs(cor(sim$X)))
  sim <- simulation(N, betaSNP, betas$blockSizes, corr, r2, minMAF=0.45)
  table(sim$X[,1])
  
})

##############################################################################
## HISTORY:
## 2015-02-07
## o simulation : add of the parameters minMAF and maxMAF
##############################################################################
