\name{cWard}
\alias{cWard}
\title{Constrained Hierarchical Clustering using the LD similarity}
\description{This function performs a constrained hierarchical clustering based on the Ward's incremental sum of squares algorithm and using the LD as a measure of similarity.}
\usage{cWard(X, blMin = 1, h = +Inf, sim = simR2, flavor = "v2", trace.time = FALSE)}
\arguments{
  \item{X}{genotype matrix coded in 1, 2 or 3. The variables to be clustered should be in columns.}
  \item{blMin}{depth of the clustering. Default 1. See details.}
  \item{h}{integer; controls the maximum lag between columns of X considered. Thus, LD measures are calculated between X[,i] and X[,j] only if i and j differ by no more than \code{h}}
  \item{sim}{a funct name specifying the LD measure to be used for the clustering. This should contain one of these funct names: simR2 or simDprime. Default: simR2}
  \item{flavor}{flavor parameter; two code versions "v1" and "v2" are implemented. "v2" is faster when h is less than the number of variables.}
  \item{trace.time}{logical. If TRUE, the CPU time in seconds used at each step of the clustering. Is returned as an attribute ("time") of the output list.}
}
\details{\code{cWard} performs a constrained hierarchical clustering of a genotype matrix, with clusters constrained by variables order. It returns an object of class \code{\link[stats]{hclust}} which can be plotted and interrogated.

The standard agglomerative hierarchical clustering starts with p groups of size 1 and successively merges the pair of groups leading to the minimal increase in within-group dispersion. This merging process is repeated until \code{blMin} groups remain.

The LD measures are calculated using the function \code{\link[snpStats]{ld}} which handles missing values of the genotype.}
\value{An object of class \code{hclust} which describes the tree produced by the clustering process. The \code{(p-1) x 2} matrix of the within dispersion measures \eqn{W_k} at each step k of the clustering are also added to the returned list.}
\references{D. Clayton. snpStats: SnpMatrix and XSnpMatrix classes and methods, 2013. R package version 1.12.0.

J. H. Ward Jr. Hierarchical grouping to optimize an objective function. Journal of the American statistical association, 58(301):236-244, 1963.}
\author{Alia Dehman, Christophe Ambroise and Pierre Neuvial}



\seealso{\code{\link{gapStatistic}}, \code{\link{select}}.}
\examples{

set.seed(5)
## unstructured data
N <- 100
p <- 50
X <- matrix(ceiling(runif(N*p)*3), N, p)
unstruct.R2 <- cWard(X, sim=simR2)
plot(unstruct.R2, main="Dendrogram")

## structured data
N <- 100
r2 <- 0.5
corr <- 0.6
p <- 120
blockSizes <- c(1:15)
sig.blocks <- 5
nb.per.block <- 1
betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2, minMAF=0.45)
Z <- sim$X
struct.R2 <- cWard(Z, sim=simR2)
struct.Dp <- cWard(Z, sim=simDp)
par(mfrow=c(1,2))
plot(unstruct.R2, main="Unstructured data")
## the underlying structure can be observed in the dendrogram 
plot(struct.R2, main="Structured data")     

groups <- betas$betaMat[,"groups"]
nBlocks <- max(groups)
groupsR2 <- cutree(struct.R2, nBlocks)
names(groupsR2) <- NULL
groupsDp <- cutree(struct.Dp, nBlocks)
names(groupsDp) <- NULL
identical(groups, as.numeric(groupsDp))   ## not always TRUE if corr<0.5 !
identical(groups, as.numeric(groupsR2))  ## not always TRUE if corr<0.5 !

}
