\name{gapStatistic}
\alias{gapStatistic}
\title{Gap Statistic for Estimating the Number of Clusters}
\description{Given a genotype matrix, this function performs a constrained hierarchical clustering of the variables and infers the optimal number of clusters using the Gap statistic method.               }
\usage{gapStatistic(X, min.nc = 1, max.nc = ncol(X) - 1, traceWB = NULL, 
    B = 500, flv = c("sample", "sampleX"), smooth = FALSE, ...)}
\arguments{
  \item{X}{genotype matrix coded in 1, 2 or 3. The variables to be clustered should be in columns.}
  \item{min.nc}{integer; the minimum number of clusters to consider. Defaults to 1.}
  \item{max.nc}{integer; the maximum number of clusters to consider. Must be at least 2 and at most (p-1) with p the number of variables to be clustered.}
  \item{traceWB}{An optional \code{(p-1) x B} matrix, giving within-cluster dispersion measures for B reference data sets.  Such measures can be obtained using  \code{\link{getTraceW}} for a given reference data set.}
  \item{B}{integer; number of the bootstrap samples. Defaults to 500.}
  \item{flv}{string; either "sample" or "sampleX". If flv="sample", each column of a reference data set is drawn independently from a uniform distribution over the values {1, 2, 3}. Else if flv="sampleX", each reference data set is resampled over the range of observed values (of X) for this column. Default "sample",}
  \item{smooth}{perform local smoothing of gap statistic (using \code{\link[stats]{smooth}})}
  \item{\dots}{optionally further arguments for the function \code{\link{cWard}}.}
}

\value{A list with elements
\item{stats}{a matrix with \code{max.nc-min.nc+1} rows and 3 columns named "E.Wks", "SE.Wks" and "W" corresponding respectively to the mean and the standard deviation of the reference data and the within-group dispersion measures at each step of clustering.}
\item{best.k}{estimated number of clusters.}
\item{tree}{object of class \code{hclust} related to the clustering of X.
}}
\references{R. Tibshirani, G. Walther, and T. Hastie. Estimating the number of clusters in a data set via the gap statistic. Journal of the Royal Statistical Society: Series B (Statistical Methodology), 63(2):411-423, 2001.}
\author{Alia Dehman, Christophe Ambroise and Pierre Neuvial}



\seealso{\code{\link{hclust}}, \code{\link{cWard}}, \code{\link{grplassoCWard}}.}
\examples{
set.seed(5)

N <- 100
p <- 30

## 1- No group structure: k=1 should be optimal
## we use B=50 to keep it fast..
X <- matrix(ceiling(runif(N*p)*3), N, p)
gapStatistic(X, B=50)$best.k

## 2- Structured data: 6 clusters
r2 <- 0.2
corr <- 0.7
blockSizes <- c(2:6, 5)
p <- sum(blockSizes)
sig.blocks <- 5
nb.per.block <- 1
betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
sim <- simulation(N, betas$betaMat[,"betaSNP"], betas$blockSizes, corr, r2, minMAF=0.45)
Z <- sim$X
ncs <- 1:10
gapS <- gapStatistic(Z, min.nc=min(ncs), max.nc=max(ncs), B=50)
print(gapS$best.k)

## plots like in Tibshirani et. al. 2001
matplot(gapS$stats[, c(3, 1)], pch=c("O", "E"), lty=1, type="b",
        ylab="Observed vs Expected Wk", xlab="number of clusters k")

gapK <- gapS$stats[,3]-gapS$stats[,1]
plot(ncs, gapK, type="o", lty=1, pch=20,
     ylab="Gap", xlab="number of clusters k")

## externalization of the calculation of trace(W)
B <- 50
traceWB <- replicate(B, getTraceW(N, p))
gapS <- gapStatistic(Z, min.nc=min(ncs), max.nc=max(ncs), traceWB=traceWB)
}
