\name{simulation}
\alias{simulation}
\title{SNP data generation}
\description{This function simulates SNP data with genotypes coded by 1, 2 and 3 as well as a quantitative phenotype assuming group-structure among the SNPs. }
\usage{simulation(N, beta, blockSizes, corr, r2, minMAF = 0.05, maxMAF = 0.5, 
    beta0 = 3)}
\arguments{
  \item{N}{integer; number of individuals.}
  \item{beta}{numeric vector;  regression coefficients.}
  \item{blockSizes}{a vector of integers specifying the sizes of the groups. The sum of the sizes must be equal to the length of \code{beta}.}
  \item{corr}{numeric value between 0 and 1. Correlation level among the variables within each block.}
  \item{r2}{numeric value between 0 and 1. Coefficient of determination}
  \item{minMAF}{minimum value of the Minor Allele Frequency. Default \eqn{0.05}. See details below.}
  \item{maxMAF}{maximum value of the Minor Allele Frequency. Default \eqn{0.5}. See details below.}
  \item{beta0}{numeric value; intercept coefficient. Default 3.}
}
\details{The effective coefficient of determination is estimated for the simulated regression model.}
\value{A list with elements
\item{y}{Vector of quantitative phenotype.}
\item{X}{\code{N x p} genotype matrix coded in \{1,2,3\} where \code{p=length(beta)} is the number of variables (SNPs).}
\item{r2}{Estimated coefficient of determination.
}}
\references{T. T. Wu, Y. F. Chen, T. Hastie, E. Sobel, and K. Lange. Genome-wide association analysis by lasso penalized logistic regression. Bioinformatics, 25(6):714-721, 2009.}
\author{Alia Dehman, Christophe Ambroise and Pierre Neuvial}



\seealso{\code{\link{simBeta}}.}
\examples{

set.seed(5)
N <- 100
r2 <- 0.3
corr <- 0.5
blockSizes <- sample(5:80, 10, replace=TRUE)
p <- sum(blockSizes)
p
sig.blocks <- 20
l <- length(sig.blocks)
if (sum(is.element(sig.blocks,blockSizes))!=l) stop("sig.blocks must be a subset of blockSizes !")
nb.per.block <- 1
betas <- simBeta(blockSizes, sig.blocks, nb.per.block)
## effective blockSizes used for simulation !
betas$blockSizes
betaSNP <- betas$betaMat[, "betaSNP"]
sim <- simulation(N, betaSNP, betas$blockSizes, corr, r2)
sim$r2 ## estimated coefficient of determination.
## The greater are the values of \code{N} and \code{p},
## the more accurate is the estimation of \code{r2}.
table(sim$X[,1])
## blockwise correlation
image(abs(cor(sim$X)))
sim <- simulation(N, betaSNP, betas$blockSizes, corr, r2, minMAF=0.45)
table(sim$X[,1])

}
